'use strict';

const upath = require('upath');
const ufo = require('ufo');
const fsExtra = require('fs-extra');

var name = "@nuxtjs/composition-api";
var version = "0.23.3";

function isFullStatic(options) {
  var _a;
  return !options.dev && !options._legacyGenerate && options.target === "static" && ((_a = options.render) == null ? void 0 : _a.ssr);
}
function isUrl(url) {
  return ["http", "//"].some((str) => url.startsWith(str));
}
function resolveRelativePath(...path) {
  return upath.resolve(__dirname, ...path);
}
function addResolvedTemplate(template, options = {}) {
  const nuxtOptions = this.nuxt.options;
  const src = resolveRelativePath(upath.join("runtime/templates", template));
  const {dst} = this.addTemplate({
    src,
    fileName: upath.join("composition-api", upath.basename(src)),
    options
  });
  const templatePath = upath.join(nuxtOptions.buildDir, dst);
  return templatePath;
}
function resolveCoreJsVersion() {
  let corejsPolyfill = this.nuxt.options.build.corejs ? String(this.nuxt.options.build.corejs) : void 0;
  try {
    if (!["2", "3"].includes(corejsPolyfill || "")) {
      const corejsPkg = this.nuxt.resolver.requireModule("core-js/package.json");
      corejsPolyfill = corejsPkg.version.slice(0, 1);
    }
  } catch (e) {
    corejsPolyfill = void 0;
  }
  return corejsPolyfill;
}
function getNuxtGlobals() {
  const nuxtOptions = this.nuxt.options;
  const globalName = nuxtOptions.globalName;
  const globalContextFactory = nuxtOptions.globals.context || ((globalName2) => `__${globalName2.toUpperCase()}__`);
  const globalNuxtFactory = nuxtOptions.globals.nuxt || ((globalName2) => `$${globalName2}`);
  const globalContext = globalContextFactory(globalName);
  const globalNuxt = globalNuxtFactory(globalName);
  return {globalContext, globalNuxt};
}

function registerBabelPlugin() {
  const nuxtOptions = this.nuxt.options;
  nuxtOptions.build.babel = nuxtOptions.build.babel || {};
  nuxtOptions.build.babel.plugins = nuxtOptions.build.babel.plugins || [];
  if (nuxtOptions.build.babel.plugins instanceof Function) {
    console.warn("Unable to automatically add Babel plugin. Make sure your custom `build.babel.plugins` returns `@nuxtjs/composition-api/dist/babel-plugin`");
  } else {
    nuxtOptions.build.babel.plugins.push(resolveRelativePath("babel-plugin"));
  }
  const actualPresets = nuxtOptions.build.babel.presets;
  nuxtOptions.build.babel.presets = (env, [defaultPreset, defaultOptions]) => {
    const newOptions = {
      ...defaultOptions,
      jsx: {
        ...typeof defaultOptions.jsx === "object" ? defaultOptions.jsx : {},
        compositionAPI: true
      }
    };
    if (typeof actualPresets === "function") {
      return actualPresets(env, [defaultPreset, newOptions]);
    }
    return [[defaultPreset, newOptions]];
  };
}

function prepareUseStatic() {
  const nuxtOptions = this.nuxt.options;
  const staticPath = upath.join(nuxtOptions.buildDir, "static-json");
  this.nuxt.hook("builder:prepared", () => {
    fsExtra.mkdirpSync(staticPath);
  });
  this.nuxt.hook("generate:route", () => {
    fsExtra.mkdirpSync(staticPath);
  });
  this.nuxt.hook("generate:done", async (generator) => {
    if (!fsExtra.existsSync(staticPath))
      return;
    const {distPath} = generator;
    fsExtra.readdirSync(staticPath).forEach((file) => fsExtra.copyFileSync(upath.join(staticPath, file), upath.join(distPath, file)));
  });
  return {staticPath};
}

function addGlobalsFile() {
  const nuxtOptions = this.options;
  const {staticPath} = prepareUseStatic.call(this);
  const {globalContext, globalNuxt} = getNuxtGlobals.call(this);
  const routerBase = ufo.withTrailingSlash(nuxtOptions.router.base);
  const publicPath = ufo.withTrailingSlash(nuxtOptions.build.publicPath);
  const globals = {
    isFullStatic: isFullStatic(nuxtOptions),
    staticPath,
    publicPath: isUrl(publicPath) ? publicPath : routerBase,
    globalContext,
    globalNuxt
  };
  const contents = Object.entries(globals).map(([key, value]) => `export const ${key} = ${JSON.stringify(value)}`).join("\n");
  const globalsFile = addResolvedTemplate.call(this, "globals.js", {
    contents
  });
  nuxtOptions.alias["@nuxtjs/composition-api/dist/globals"] = globalsFile;
}

const compositionApiModule = function compositionApiModule2() {
  const nuxtOptions = this.nuxt.options;
  addGlobalsFile.call(this);
  nuxtOptions.alias["~composition-api"] = upath.resolve(__dirname, "index.mjs");
  nuxtOptions.build.transpile = nuxtOptions.build.transpile || [];
  nuxtOptions.build.transpile.push("@nuxtjs/composition-api", "~composition-api", __dirname);
  nuxtOptions.alias["@vue/composition-api"] = this.nuxt.resolver.resolveModule("@vue/composition-api/dist/vue-composition-api.esm.js");
  this.extendBuild((config) => {
    if (!config.module)
      return;
    config.module.rules.unshift({
      test: /\.mjs$/,
      type: "javascript/auto",
      include: [/node_modules/]
    });
  });
  if (nuxtOptions.features.middleware) {
    const middleware = addResolvedTemplate.call(this, "register.js");
    this.nuxt.hook("build:templates", ({templateVars}) => {
      templateVars.middleware.unshift({
        src: middleware,
        dst: "." + upath.sep + upath.relative(nuxtOptions.buildDir, middleware),
        name: "compositionApiRegistration"
      });
    });
  } else if (nuxtOptions.features.layouts) {
    this.addLayout(resolveRelativePath("runtime/templates/layout.js"), "0");
  } else {
    const dst = addResolvedTemplate.call(this, "register.js");
    this.nuxt.hook("modules:done", () => this.nuxt.hook("build:before", () => nuxtOptions.plugins.unshift(dst)));
  }
  this.nuxt.hook("vite:extend", async (ctx) => {
    const {compositionApiPlugin} = await Promise.resolve().then(function () { return require('./module.js-vite-plugin.js'); });
    ctx.config.plugins.push(compositionApiPlugin());
  });
  registerBabelPlugin.call(this);
  addResolvedTemplate.call(this, "polyfill.client.js", {
    corejsPolyfill: resolveCoreJsVersion.call(this)
  });
  const globalPlugin = addResolvedTemplate.call(this, "plugin.js");
  this.nuxt.hook("modules:done", () => {
    nuxtOptions.plugins.unshift(globalPlugin);
  });
  if (!nuxtOptions.buildModules.includes("@nuxtjs/pwa") && !nuxtOptions.modules.includes("@nuxtjs/pwa")) {
    nuxtOptions.plugins.push(addResolvedTemplate.call(this, "meta.js"));
  } else if (nuxtOptions.dev) {
    console.warn("useMeta is not supported in onGlobalSetup as @nuxtjs/pwa detected.\nSee https://github.com/nuxt-community/composition-api/issues/307");
  }
};
compositionApiModule.meta = {
  name,
  version
};

module.exports = compositionApiModule;
